/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.se.validation.service;

import java.io.IOException;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;

import org.apache.commons.lang.CharEncoding;
import org.ow2.petals.commons.stream.InputStreamForker;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.ow2.petals.se.validation.Constants;
import org.xml.sax.SAXException;

/**
 * The VALIDATION service
 * 
 * @author alouis - EBM WebSourcing
 * @author mbeldame - EBM WebSourcing
 * 
 */
public class ValidationService {

    private final Logger logger;

    public ValidationService(Logger logger) {
        super();
        this.logger = logger;
    }

    /**
     * Set a Fault for the filter operation.
     * 
     * @param exchange
     * @param message
     * @throws MessagingException
     * @throws PEtALSCDKException
     * @throws JBIException
     */
    private static final void setFilterFault(Exchange exchange, String message)
            throws MessagingException, PEtALSCDKException {
        final StringBuffer faultMessage = new StringBuffer();
        faultMessage
                .append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><tns:validationFault xmlns:tns='");
        faultMessage.append(Constants.VALIDATION_NS);
        faultMessage.append("'><tns:message>");
        faultMessage.append(message);
        faultMessage.append("</tns:message></tns:validationFault>");
        final Fault fault = exchange.createFault();
        fault.setContent(UtilFactory.getSourceUtil().createSource(faultMessage.toString(),
                CharEncoding.UTF_8));
        exchange.setFault(fault);

    }

    /**
     * Set the Ko response for the validate operation.
     * 
     * @param exchange
     * @param message
     * @throws PEtALSCDKException
     * @throws MessagingException
     */
    private static final void setValidateResponseKo(Exchange exchange, String message)
            throws MessagingException, PEtALSCDKException {
        final StringBuffer responseMessage = new StringBuffer();
        responseMessage
                .append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><tns:validateResponse xmlns:tns='");
        responseMessage.append(Constants.VALIDATION_NS);
        responseMessage.append("'><tns:valid>false</tns:valid><tns:comment>");
        responseMessage.append(message);
        responseMessage.append("</tns:comment></tns:validateResponse>");
        exchange.setOutMessageContent(UtilFactory.getSourceUtil().createSource(
                responseMessage.toString(), CharEncoding.UTF_8));
    }

    /**
     * Set the Ok response for the validate operation.
     * 
     * @param exchange
     * @param message
     * @throws PEtALSCDKException
     * @throws MessagingException
     */
    private static final void setValidateResponseOk(Exchange exchange) throws MessagingException,
            PEtALSCDKException {
        final StringBuffer responseMessage = new StringBuffer();
        responseMessage
                .append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><tns:validateResponse xmlns:tns='");
        responseMessage.append(Constants.VALIDATION_NS);
        responseMessage.append("'><tns:valid>true</tns:valid></tns:validateResponse>");
        exchange.setOutMessageContent(UtilFactory.getSourceUtil().createSource(
                responseMessage.toString(), CharEncoding.UTF_8));
    }

    /**
     * Filter operation : if message is valid, it is forwarded on the OUT
     * Otherwise, a Fault is returned.
     * 
     * @param exchange
     * @param xsdSchema
     * @throws JBIException
     */
    public void filter(Exchange exchange, Schema xsdSchema) throws JBIException {
        Source responseSource = null;

        Source source = exchange.getInMessageContentAsSource();
        if (source instanceof StreamSource) {
            final InputStreamForker messageAsStream = new InputStreamForker(((StreamSource) source)
                    .getInputStream());
            source = new StreamSource(messageAsStream.getInputStreamOne());
            responseSource = new StreamSource(messageAsStream.getInputStreamTwo());
        } else {
            responseSource = source;
        }

        try {
            xsdSchema.newValidator().validate(source);
            exchange.setOutMessageContent(responseSource);
            this.logger.info("XML message is valid");
        } catch (final SAXException e) {
            this.logger.warning("XML message is invalid : " + e.getMessage());
            setFilterFault(exchange, e.getMessage());
        } catch (final IOException e) {
            setFilterFault(exchange, e.getMessage());
        }
    }

    /**
     * Validate operation; If message is valid, it is forwarded on the OUT
     * Otherwise, a Fault is returned
     * 
     * @param exchange
     * @throws MessagingException
     * @throws PEtALSCDKException
     */
    public void validate(Exchange exchange, Schema xsdSchema) throws MessagingException,
            PEtALSCDKException {

        // Get in Message content
        final Source inContent = exchange.getInMessageContentAsSource();

        try {
            xsdSchema.newValidator().validate(inContent);
            this.logger.info("XML message is valid");
            setValidateResponseOk(exchange);
        } catch (final SAXException e) {
            this.logger.warning("XML message is invalid : " + e.getMessage());
            setValidateResponseKo(exchange, e.getMessage());
        } catch (final IOException e) {
            this.logger.warning("XML message is invalid : " + e.getMessage());
            setValidateResponseKo(exchange, e.getMessage());
        }
    }
}
